precision mediump float;

varying highp vec2 textureCoordinate;
uniform sampler2D videoFrame;   //视频预览纹理
uniform sampler2D coverFrame;   //cover纹理
uniform int     _ispreview;     //是否预览图像
uniform int     _middletype;     //0：椭圆形，1：矩形
uniform vec2    _middlepoint;   //焦点中心
uniform vec2    _range;         //焦点区域横向和纵向的半径
uniform int     _type;           //特效类型
uniform int     _depthenable;    //是否打开景深处理,1为打开

vec4 _maskcolor = vec4(1.0,1.0,1.0,0.0);    //预览遮盖色彩
vec2 _framesize = vec2(480.0,320.0);        //完整区域


uniform vec2    _degree;         //角度,景深矩形区域偏转，尚未实现


/*
 ** Photoshop & misc math
 ** Blending modes, RGB/HSL/Contrast/Desaturate, levels control
 **
 ** Romain Dura | Romz
 ** Blog: http://blog.mouaif.org
 ** Post: http://blog.mouaif.org/?p=94
 */


/*
 ** Desaturation
 */

vec4 Desaturate(vec3 color, float Desaturation)
{
	vec3 grayXfer = vec3(0.3, 0.59, 0.11);
	vec3 gray = vec3(dot(grayXfer, color));
	return vec4(mix(color, gray, Desaturation), 1.0);
}


/*
 ** Hue, saturation, luminance
 */

vec3 RGBToHSL(vec3 color)
{
	vec3 hsl; // init to 0 to avoid warnings ? (and reverse if + remove first part)
	
	float fmin = min(min(color.r, color.g), color.b);    //Min. value of RGB
	float fmax = max(max(color.r, color.g), color.b);    //Max. value of RGB
	float delta = fmax - fmin;             //Delta RGB value
    
	hsl.z = (fmax + fmin) / 2.0; // Luminance
    
	if (delta == 0.0)		//This is a gray, no chroma...
	{
		hsl.x = 0.0;	// Hue
		hsl.y = 0.0;	// Saturation
	}
	else                                    //Chromatic data...
	{
		if (hsl.z < 0.5)
        hsl.y = delta / (fmax + fmin); // Saturation
		else
        hsl.y = delta / (2.0 - fmax - fmin); // Saturation
		
		float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
		float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
		float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;
        
		if (color.r == fmax )
        hsl.x = deltaB - deltaG; // Hue
		else if (color.g == fmax)
        hsl.x = (1.0 / 3.0) + deltaR - deltaB; // Hue
		else if (color.b == fmax)
        hsl.x = (2.0 / 3.0) + deltaG - deltaR; // Hue
        
		if (hsl.x < 0.0)
        hsl.x += 1.0; // Hue
		else if (hsl.x > 1.0)
        hsl.x -= 1.0; // Hue
	}
    
	return hsl;
}

float HueToRGB(float f1, float f2, float hue)
{
	if (hue < 0.0)
    hue += 1.0;
	else if (hue > 1.0)
    hue -= 1.0;
	float res;
	if ((6.0 * hue) < 1.0)
    res = f1 + (f2 - f1) * 6.0 * hue;
	else if ((2.0 * hue) < 1.0)
    res = f2;
	else if ((3.0 * hue) < 2.0)
    res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
	else
    res = f1;
	return res;
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb;
	
	if (hsl.y == 0.0)
    rgb = vec3(hsl.z); // Luminance
	else
	{
		float f2;
		
		if (hsl.z < 0.5)
        f2 = hsl.z * (1.0 + hsl.y);
		else
        f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
        
		float f1 = 2.0 * hsl.z - f2;
		
		rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
		rgb.g = HueToRGB(f1, f2, hsl.x);
		rgb.b= HueToRGB(f1, f2, hsl.x - (1.0/3.0));
	}
	
	return rgb;
}


/*
 ** Contrast, saturation, brightness
 ** Code of this function is from TGM's shader pack
 ** http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=21057
 */

// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	const float AvgLumR = 0.5;
	const float AvgLumG = 0.5;
	const float AvgLumB = 0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	vec3 conColor = mix(AvgLumin, satColor, con);
	return conColor;
}


/*
 ** Float blending modes
 ** Adapted from here: http://www.nathanm.com/photoshop-blending-math/
 ** But I modified the HardMix (wrong condition), Overlay, SoftLight, ColorDodge, ColorBurn, VividLight, PinLight (inverted layers) ones to have correct results
 */

#define BlendLinearDodgef 			BlendAddf
#define BlendLinearBurnf 			BlendSubstractf
#define BlendAddf(base, blend) 		min(base + blend, 1.0)
#define BlendSubstractf(base, blend) 	max(base + blend - 1.0, 0.0)
#define BlendLightenf(base, blend) 		max(blend, base)
#define BlendDarkenf(base, blend) 		min(blend, base)
#define BlendLinearLightf(base, blend) 	(blend < 0.5 ? BlendLinearBurnf(base, (2.0 * blend)) : BlendLinearDodgef(base, (2.0 * (blend - 0.5))))
#define BlendScreenf(base, blend) 		(1.0 - ((1.0 - base) * (1.0 - blend)))
#define BlendOverlayf(base, blend) 	(base < 0.5 ? (2.0 * base * blend) : (1.0 - 2.0 * (1.0 - base) * (1.0 - blend)))
#define BlendSoftLightf(base, blend) 	((blend < 0.5) ? (2.0 * base * blend + base * base * (1.0 - 2.0 * blend)) : (sqrt(base) * (2.0 * blend - 1.0) + 2.0 * base * (1.0 - blend)))
#define BlendColorDodgef(base, blend) 	((blend == 1.0) ? blend : min(base / (1.0 - blend), 1.0))
#define BlendColorBurnf(base, blend) 	((blend == 0.0) ? blend : max((1.0 - ((1.0 - base) / blend)), 0.0))
#define BlendVividLightf(base, blend) 	((blend < 0.5) ? BlendColorBurnf(base, (2.0 * blend)) : BlendColorDodgef(base, (2.0 * (blend - 0.5))))
#define BlendPinLightf(base, blend) 	((blend < 0.5) ? BlendDarkenf(base, (2.0 * blend)) : BlendLightenf(base, (2.0 *(blend - 0.5))))
#define BlendHardMixf(base, blend) 	((BlendVividLightf(base, blend) < 0.5) ? 0.0 : 1.0)
#define BlendReflectf(base, blend) 		((blend == 1.0) ? blend : min(base * base / (1.0 - blend), 1.0))


/*
 ** Vector3 blending modes
 */

// Component wise blending
#define Blend(base, blend, funcf) 		vec3(funcf(base.r, blend.r), funcf(base.g, blend.g), funcf(base.b, blend.b))

#define BlendNormal(base, blend) 		(blend)
#define BlendLighten				BlendLightenf
#define BlendDarken				BlendDarkenf
#define BlendMultiply(base, blend) 		(base * blend)
#define BlendAverage(base, blend) 		((base + blend) / 2.0)
#define BlendAdd(base, blend) 		min(base + blend, vec3(1.0))
#define BlendSubstract(base, blend) 	max(base + blend - vec3(1.0), vec3(0.0))
#define BlendDifference(base, blend) 	abs(base - blend)
#define BlendNegation(base, blend) 	(vec3(1.0) - abs(vec3(1.0) - base - blend))
#define BlendExclusion(base, blend) 	(base + blend - 2.0 * base * blend)
#define BlendScreen(base, blend) 		Blend(base, blend, BlendScreenf)
#define BlendOverlay(base, blend) 		Blend(base, blend, BlendOverlayf)
#define BlendSoftLight(base, blend) 	Blend(base, blend, BlendSoftLightf)
#define BlendHardLight(base, blend) 	BlendOverlay(blend, base)
#define BlendColorDodge(base, blend) 	Blend(base, blend, BlendColorDodgef)
#define BlendColorBurn(base, blend) 	Blend(base, blend, BlendColorBurnf)
#define BlendLinearDodge			BlendAdd
#define BlendLinearBurn			BlendSubstract
// Linear Light is another contrast-increasing mode
// If the blend color is darker than midgray, Linear Light darkens the image by decreasing the brightness. If the blend color is lighter than midgray, the result is a brighter image due to increased brightness.
#define BlendLinearLight(base, blend) 	Blend(base, blend, BlendLinearLightf)
#define BlendVividLight(base, blend) 	Blend(base, blend, BlendVividLightf)
#define BlendPinLight(base, blend) 		Blend(base, blend, BlendPinLightf)
#define BlendHardMix(base, blend) 		Blend(base, blend, BlendHardMixf)
#define BlendReflect(base, blend) 		Blend(base, blend, BlendReflectf)
#define BlendGlow(base, blend) 		BlendReflect(blend, base)
#define BlendPhoenix(base, blend) 		(min(base, blend) - max(base, blend) + vec3(1.0))
#define BlendOpacity(base, blend, F, O) 	(F(base, blend) * O + blend * (1.0 - O))


// Hue Blend mode creates the result color by combining the luminance and saturation of the base color with the hue of the blend color.
vec3 BlendHue(vec3 base, vec3 blend)
{
	vec3 baseHSL = RGBToHSL(base);
	return HSLToRGB(vec3(RGBToHSL(blend).r, baseHSL.g, baseHSL.b));
}

// Saturation Blend mode creates the result color by combining the luminance and hue of the base color with the saturation of the blend color.
vec3 BlendSaturation(vec3 base, vec3 blend)
{
	vec3 baseHSL = RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r, RGBToHSL(blend).g, baseHSL.b));
}

// Color Mode keeps the brightness of the base color and applies both the hue and saturation of the blend color.
vec3 BlendColor(vec3 base, vec3 blend)
{
	vec3 blendHSL = RGBToHSL(blend);
	return HSLToRGB(vec3(blendHSL.r, blendHSL.g, RGBToHSL(base).b));
}

// Luminosity Blend mode creates the result color by combining the hue and saturation of the base color with the luminance of the blend color.
vec3 BlendLuminosity(vec3 base, vec3 blend)
{
	vec3 baseHSL = RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r, baseHSL.g, RGBToHSL(blend).b));
}


/*
 ** Gamma correction
 ** Details: http://blog.mouaif.org/2009/01/22/photoshop-gamma-correction-shader/
 */

#define GammaCorrection(color, gamma)								pow(color, 1.0 / gamma)

/*
 ** Levels control (input (+gamma), output)
 ** Details: http://blog.mouaif.org/2009/01/28/levels-control-shader/
 */

#define LevelsControlInputRange(color, minInput, maxInput)				min(max(color - vec3(minInput), vec3(0.0)) / (vec3(maxInput) - vec3(minInput)), vec3(1.0))
#define LevelsControlInput(color, minInput, gamma, maxInput)				GammaCorrection(LevelsControlInputRange(color, minInput, maxInput), gamma)
#define LevelsControlOutputRange(color, minOutput, maxOutput) 			mix(vec3(minOutput), vec3(maxOutput), color)
#define LevelsControl(color, minInput, gamma, maxInput, minOutput, maxOutput) 	LevelsControlOutputRange(LevelsControlInput(color, minInput, gamma, maxInput), minOutput, maxOutput)




////////////////以下为基本参数调整///////////////


//控制数值在0-1之间
float rangeSet(in float num)
{
    float result = num;
    if(num >= 1.0)
        result  =  1.0;
    if(num <= 0.0)
        result = 0.0;
    return result;
    
}

//控制颜色值rgba在0-1之间
vec4 rangeColor(vec4 color)
{
    vec4 result = color;
    result.r = rangeSet(color.r);
    result.g = rangeSet(color.g);
    result.b = rangeSet(color.b);
    return result;
}


//对比度调整
float modiContrast(float source ,float intensity)
{
    float n;
    float contrast = intensity - 1.0;
    float result = source;
    n = (source > 0.5)?(1.0 - source):source;
    if(n < 0.0)
        n = 0.0;
    if(contrast < 0.0)
    {
        n = 0.5 * pow(2.0 * n,1.0 + contrast);
    }
    else
    {
        float power = (contrast == 1.0) ? 1.0 : (1.0/(1.0 - contrast));
        n = 0.5 * pow (2.0 * n , power);
    }
    
    result = (source > 0.5) ? (1.0 - n) :n;
    
    return result;
}

//色阶
vec4 modiColorlevel(vec4 color,float inmin,float inmax,float outmin,float outmax)
{
    vec4 result = color;
    vec3 finten;
    if(inmin != inmax)
    {
        finten.r = (result.r  - inmin) / (inmax - inmin);
        finten.g = (result.g  - inmin) / (inmax - inmin);
        finten.b = (result.b  - inmin) / (inmax - inmin);
    }
    result.r = clamp((outmin + finten.r * (outmax - outmin)),outmin,outmax);
    result.g = clamp((outmin + finten.g * (outmax - outmin)),outmin,outmax);
    result.b = clamp((outmin + finten.b * (outmax - outmin)),outmin,outmax);
    return result;
}


//饱和度调整
vec4 modiStaturation(vec4 color, float intensity)
{
    vec4 result = color;
    float modiR = 0.4086;
    float modiG = 0.5094;
    float modiB = 0.0820;
    
    float a = (1.0 - intensity) * modiR + intensity;
    float b = (1.0 - intensity) * modiR;
    float c = (1.0 - intensity) * modiR;
    
    float d = (1.0 - intensity) * modiG;
    float e = (1.0 - intensity) * modiG + intensity;
    float f = (1.0 - intensity) * modiG;
    
    float g = (1.0 - intensity) * modiB;
    float h = (1.0 - intensity) * modiB;
    float i = (1.0 - intensity) * modiB + intensity;
    
    mat4 mat = mat4(    a,      b,      c,      0.0,
                        d,      e,      f,      0.0,
                        g,      h,      i,      1.0,
                        0.0,    0.0,    0.0,    1.0);
    result = mat * color;
    
    return rangeColor(color);
}


float shadowsAdd(float source,float intensity)
{
    float result;
    result = intensity * 0.667 * (1.0 - (source-127.0) * (source - 127.0)/(127.0 * 127.0));
    return result;
}
float shadowsSub(float source,float intensity)
{
    float result;
    result = intensity * (1.075 - 1.0/((255.0 - source)/16.0 + 1.0));
    return result;
}
//颜色平衡
float colorbalance(float source,float intensity)
{
    float result = source;
    if(intensity > 0.0)
    {
        result += shadowsAdd(result,intensity);
    }
    else
    {
        result += shadowsSub(result,intensity);
    }
    return result;
}

//模糊
vec4 fuzzy(vec4 color,float intensity)
{   
    vec4 result = color;
    vec2 pTmp;
    
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    result = result * 0.2;
    
    return rangeColor(result);
}

//forst grass
vec4 forst(vec4 color,float intensity)
{
    vec4 result = color;
    vec2 pTmp;
    
    float intensity2 = intensity * 2.0;
    
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y);
    result += texture2D(coverFrame,pTmp);
    pTmp = vec2(textureCoordinate.x,textureCoordinate.y-intensity);
    result += texture2D(coverFrame,pTmp);

    pTmp = vec2(textureCoordinate.x+intensity2,textureCoordinate.y+intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity2,textureCoordinate.y-intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity2,textureCoordinate.y-intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity2,textureCoordinate.y+intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x,textureCoordinate.y+intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity2,textureCoordinate.y);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity2,textureCoordinate.y);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x,textureCoordinate.y-intensity2);
    result += texture2D(videoFrame,pTmp);

    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity2,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity2,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity2,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity2,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity2);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity2);
    result += texture2D(videoFrame,pTmp);
    
    
    result = result / 50.0;
    
    return rangeColor(result);
    
}


//景深
vec4 depth(vec4 color)
{
    vec4 result = color;
    float fPassRange=20.0;    //椭圆远景过度范围参数
    
    float fModiFuzzy = 0.003;   //离开焦点区域模糊程度参数
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = fuzzy(result,fModiFuzzy);
    
    float percent;
    vec2 tmpCood = vec2(textureCoordinate.y * _framesize.y,textureCoordinate.x * _framesize.x);
    if(_ispreview == 1)     //预览处理
    {
        if(_middletype == 1)//椭圆形
        {
            //点在圆形外部
            percent = (tmpCood.x-_middlepoint.x) * (tmpCood.x-_middlepoint.x)/(_range.x * _range.x) + (tmpCood.y-_middlepoint.y) * (tmpCood.y-_middlepoint.y)/(_range.y * _range.y) - 1.0;
            if(percent < 0.0 )
            {
                result = normalColor;
            }
            else if(percent > fPassRange)
            {
                result = normalColor + (_maskcolor - normalColor)*0.95;
            }
            else
            {
                result = normalColor + (_maskcolor - normalColor)*percent*0.95/fPassRange;    
            }
        }
        else if(_middletype == 0)//矩形
        {
            percent = abs(textureCoordinate.y-_middlepoint.y/_framesize.y) - _range.y/_framesize.y;
            if(percent < 0.0)
            {
                result = normalColor;
            }
            else if(percent >= fPassRange/_framesize.y)
            {
                result = normalColor + (_maskcolor - normalColor)*0.95;
            }
            else
            {
                result = normalColor + (_maskcolor - normalColor)*0.95 * percent/(fPassRange/_framesize.y);
            }
        }
        else
        {
            //处理默认
            color = normalColor;
        }
    }
    else      //实际处理
    {
        if(_middletype == 1)//椭圆形
        {
            //点在圆形外部
            percent = (tmpCood.x-_middlepoint.x) * (tmpCood.x-_middlepoint.x)/(_range.x * _range.x) + (tmpCood.y-_middlepoint.y) * (tmpCood.y-_middlepoint.y)/(_range.y * _range.y) - 1.0;
            
            if(percent < 0.0 )
            {
                result = normalColor;
            }
            else if(percent > fPassRange)
            {
                result = normalColor + (result - normalColor)*0.95;
            }
            else
            {
                result = normalColor + (result - normalColor)*percent*0.95/fPassRange;
            }
        }
        else if(_middletype == 0)//矩形
        {
            percent = abs(textureCoordinate.y-_middlepoint.y/_framesize.y) - _range.y/_framesize.y;
            if(percent < 0.0)
            {
                result = normalColor;
            }
            else if(percent >= fPassRange/_framesize.y)
            {
                result = normalColor + (color - normalColor)*0.95;
            }
            else
            {
                result = normalColor + (color - normalColor)*0.95 * percent/(fPassRange/_framesize.y);
            }
        }
        else
        {
            //处理默认
            color = normalColor;
        }

    }
    //过滤非法值
    return rangeColor(result);
}

vec4 blackCorner(vec4 color)
{
    vec4 result = color;
    float fPassRange=40.0;    //椭圆远景过度范围参数
    
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = color * 0.6;
    
    if((textureCoordinate.x-0.5) * (textureCoordinate.x-0.5) +(textureCoordinate.y-0.5) * (textureCoordinate.y-0.5) > 1.0)
    {
        result = result *0.8;
    }
    else
    {
        result = result;
    }
    
    //过滤非法值
    return rangeColor(result);
}

////////以下为特效处理部分////////////

//黑白
vec4 gray(vec4 color)
{
    vec4 result = color;
    float finmax = 215.0/255.0;  //色阶最大值
    float finmin = 35.0/255.0;  //色阶最小值
    
    float foutmax = 255.0/255.0;  //色阶最大值
    float foutmin = 0.0/255.0;  //色阶最小值
    
    //取灰度图
    float fgray = dot(color.rgb,vec3(0.299,0.587,0.114));
    result = vec4(fgray,fgray,fgray,color.a);

    //调整色阶
    result = modiColorlevel(result,finmin,finmax,foutmin,foutmax);

    //景深处理
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;
}

//艳丽
vec4 colord(vec4 color)
{
    vec4 result = color;
    float finmax = 235.0/255.0;
    float finmin = 50.0/255.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 10.0/255.0;
    
    float fModiBrightness=1.2;       //亮度计算控制参数范围0.5-2.0
    float fModiSaturation = 1.35;    //饱和度调整参数
    float fModiContrast = 1.0;      //对比度
    
    //调整色阶
    result = modiColorlevel(result,finmin,finmax,foutmin,foutmax);
    //调整对比度
    result.rgb =  ContrastSaturationBrightness(result.rgb, fModiBrightness, fModiSaturation, fModiContrast);
    
    //景深处理
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    
    return result;
}
//复古
vec4 old(vec4 color)
{
    vec4 result =  color;
    float changeR = 50.0/255.0;     //R调整范围
    float changeG = 6.0/255.0;      //G调整范围
    float changeB = -40.0/255.0;    //B调整范围
    
    float finmax = 235.0/255.0;           //色阶调整
    float finmin = 50.0/255.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 10.0/255.0;
        
    //调整RGB
    result.r += changeR;
    result.g += changeG;
    result.b += changeB;
    
    //调整色阶
    result = modiColorlevel(result,foutmin,foutmax,finmin,finmax);
    
    //景深处理
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;
}
vec4 blackside(vec4 color)
{
    vec4 result = color;
    
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = color * 0.6;
    
    float d = textureCoordinate.y;
	// if d > outer, then 0 -> black 
	// if d < inner, then 1 -> actual color
	// range of (outer - inner) is where the vignette occurs
    
    result  = normalColor + (result - normalColor) * smoothstep(0.4, 1.0, d);
    return result;
}



//普通
vec4 normal(vec4 color)
{
    vec4 result = color;
    if(_depthenable == 1)
    {
        result = depth(result);
    }    
    
    return result;
}
//美白
vec4 white(vec4 color)
{
    vec4 result = color;
    
    float fModiBrightness=1.2;       //亮度计算控制参数范围0.5-2.0
    float fModiSaturation = 0.6;    //饱和度调整参数
    float fModiContrast = 1.0;      //对比度
    float fModiFuzzy = 0.001;       //模糊程度调整参数


    //模糊
    result = fuzzy(result,fModiFuzzy);
    //调整参数    
    result.rgb = LevelsControl(result.rgb, 0.0, vec3(2.0), 1.0, 0.0, 1.0);
    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;
}

//优雅
vec4 elegant(vec4 color)
{
    vec4 result = color;
    
    float fgamma = 1.2;
    float finmax = 220.0/255.0;           //色阶调整
    float finmin = 0.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 20.0/255.0;

    
    float fModiBrightness=1.0;       //亮度计算控制参数范围0.5-2.0
    float fModiSaturation = 1.0;    //饱和度调整参数
    float fModiContrast = 0.9;      //对比度

    vec3  vModiColorbalance = vec3(20.0/255.0,-5.0/255.0,-10.0/255.0);

    //调整色阶
    result.rgb = LevelsControl(result.rgb,finmin,vec3(fgamma),finmax,foutmin,foutmax);
    
    //color balance
    result.r = colorbalance(result.r,vModiColorbalance.r);
    result.g = colorbalance(result.g,vModiColorbalance.g);
    result.b = colorbalance(result.b,vModiColorbalance.b);
    
    //调整亮度
    result.b += result.b * 0.2;
    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }


    return result;
}


//唯美
vec4 beauty(vec4 color)
{
    vec4 result = color;
    
    float finmax = 240.0/255.0;           //色阶调整
    float finmin = 0.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 30.0/255.0;

    float fModiContrast = 0.45;

    vec3  vModiColorbalance = vec3(20.0/255.0,15.0/255.0,-25.0/255.0);
    
    //亮度调整
    result.b += result.b * 0.2;
    
    //调整对比度
    result.b = modiContrast(result.b,fModiContrast);
    
    //调整色阶
    result.rgb = LevelsControl(result.rgb,finmin,vec3(1.0),finmax,foutmin,foutmax);

    //color balance
    result.r = colorbalance(result.r,vModiColorbalance.r);
    result.g = colorbalance(result.g,vModiColorbalance.g);
    result.b = colorbalance(result.b,vModiColorbalance.b);
    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;    
}
//清新
vec4 fresh(vec4 color)
{
    vec4 result = color;
    float fModiContrast = 0.4;
    vec3  vModiColorbalance = vec3(75.0/255.0,15.0/255.0,-10.0/255.0);
    float fgamma = 1.25;
    
    //调整对比度
    result.b = modiContrast(result.b,fModiContrast);
    
    //color balance
    result.r = colorbalance(result.r,vModiColorbalance.r);
    result.g = colorbalance(result.g,vModiColorbalance.g);
    result.b = colorbalance(result.b,vModiColorbalance.b);


    //调整色阶
    result.rgb = LevelsControl(result.rgb,0.0,vec3(fgamma),1.0,0.0,1.0); 

    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;
}
//dream效果
vec4 dream(vec4 color)
{
    vec4 result = color;
    result += texture2D(coverFrame,textureCoordinate);
    result = blackCorner(result);
    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }

    return result;
}


//光照效果
vec4 light(vec4 color)
{
    return color;
}

/////////////////主程序////////////////////
void main()
{
    vec4 calculatedColor;
    calculatedColor = texture2D(videoFrame,textureCoordinate);
    //图像调整运算过程
    if(_type == 0)
    {
        //tese light
        //calculatedColor = dream(calculatedColor);
    }
    if(_type == 1) //美白
    {
        calculatedColor = white(calculatedColor);
    }
    else if(_type == 2) //复古
    {
        calculatedColor = old(calculatedColor);
    }
    else if(_type == 3) //艳丽lomo
    {
        calculatedColor = colord(calculatedColor);
    }
    else if(_type == 4) //黑白
    {
        calculatedColor = gray(calculatedColor);
    }
    else if(_type == 5) //优雅
    {
        calculatedColor = elegant(calculatedColor);
    }
    else if(_type == 6) //唯美
    {
        calculatedColor = beauty(calculatedColor);
    }
    else if(_type == 7) //清新
    {
        calculatedColor = fresh(calculatedColor);
    }
    else if(_type == 8)//dream
    {
        calculatedColor = dream(calculatedColor);
    }
    calculatedColor = forst(calculatedColor,0.005);
    gl_FragColor = calculatedColor;
}


