precision mediump float;

varying highp vec2 textureCoordinate;
uniform sampler2D videoFrame;   //视频预览纹理
uniform sampler2D coverFrame;   //cover纹理
uniform int     _ispreview;     //是否预览图像
uniform int     _middletype;     //0：椭圆形，1：矩形
uniform vec2    _middlepoint;   //焦点中心
uniform vec2    _range;         //焦点区域横向和纵向的半径
uniform int     _type;           //特效类型
uniform int     _depthenable;    //是否打开景深处理,1为打开

vec4 _maskcolor = vec4(1.0,1.0,1.0,0.0);    //预览遮盖色彩
vec2 _framesize = vec2(480.0,320.0);        //完整区域


uniform vec2    _degree;         //角度,景深矩形区域偏转，尚未实现

#define GammaCorrection(color, gamma)								pow(color, 1.0 / gamma)
#define LevelsControlInputRange(color, minInput, maxInput)				min(max(color - vec3(minInput), vec3(0.0)) / (vec3(maxInput) - vec3(minInput)), vec3(1.0))
#define LevelsControlInput(color, minInput, gamma, maxInput)				GammaCorrection(LevelsControlInputRange(color, minInput, maxInput), gamma)
#define LevelsControlOutputRange(color, minOutput, maxOutput) 			mix(vec3(minOutput), vec3(maxOutput), color)
#define LevelsControl(color, minInput, gamma, maxInput, minOutput, maxOutput) 	LevelsControlOutputRange(LevelsControlInput(color, minInput, gamma, maxInput), minOutput, maxOutput)
//模糊
vec4 fuzzy(vec4 color,float intensity)
{   
    vec4 result = color;
    vec2 pTmp;
    
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    result = result * 0.2;
    
    return result;
}

//景深
vec4 depth(vec4 color)
{
    vec4 result = color;
    float fPassRange=80.0;    //椭圆远景过度范围参数
    
    float fModiFuzzy = 0.002;   //离开焦点区域模糊程度参数
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = fuzzy(result,fModiFuzzy);
    
    float distance;
    vec2 tmpCood = vec2(textureCoordinate.y * _framesize.y,textureCoordinate.x * _framesize.x);
    if(_ispreview == 1)     //预览处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            color = normalColor;
        }
    }
    else      //实际处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            result = normalColor;
        }
        
    }
    return result;
}

float shadowsAdd(float source,float intensity)
{
    float result;
    result = intensity * 0.667 * (1.0 - (source-127.0) * (source - 127.0)/(127.0 * 127.0));
    return result;
}
float shadowsSub(float source,float intensity)
{
    float result;
    result = intensity * (1.075 - 1.0/((255.0 - source)/16.0 + 1.0));
    return result;
}
//颜色平衡
float colorbalance(float source,float intensity)
{
    float result = source;
    if(intensity > 0.0)
    {
        result += shadowsAdd(result,intensity);
    }
    else
    {
        result += shadowsSub(result,intensity);
    }
    return result;
}

//对比度调整
float modiContrast(float source ,float intensity)
{
    float n;
    float contrast = intensity - 1.0;
    float result = source;
    n = (source > 0.5)?(1.0 - source):source;
    if(n < 0.0)
        n = 0.0;
    if(contrast < 0.0)
    {
        n = 0.5 * pow(2.0 * n,1.0 + contrast);
    }
    else
    {
        float power = (contrast == 1.0) ? 1.0 : (1.0/(1.0 - contrast));
        n = 0.5 * pow (2.0 * n , power);
    }
    
    result = (source > 0.5) ? (1.0 - n) :n;
    
    return result;
}

vec4 blackCorner(vec4 color)
{
    vec4 result = color;
    
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = color * 0.3;
    
    float d = distance(textureCoordinate, vec2(0.5,0.5));
	// if d > outer, then 0 -> black 
	// if d < inner, then 1 -> actual color
	// range of (outer - inner) is where the vignette occurs
    
    result  = normalColor + (result - normalColor) * smoothstep(0.5, 0.8, d);
    return result;
}


//唯美
vec4 beauty(vec4 color)
{
    vec4 result = color;
    
    float finmax = 240.0/255.0;           //色阶调整
    float finmin = 0.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 30.0/255.0;
    
    float fModiContrast = 0.45;
    
    vec3  vModiColorbalance = vec3(50.0/255.0,35.0/255.0,-45.0/255.0);

    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }
    
    
    //亮度调整
    //result.b = result.b * 0.45;
    
    //调整对比度
    result.b = modiContrast(result.b,fModiContrast);
    
    //调整色阶
    result.rgb = LevelsControl(result.rgb,finmin,vec3(1.0),finmax,foutmin,foutmax);
    
    //color balance
    result.r = colorbalance(result.r,vModiColorbalance.r);
    result.g = colorbalance(result.g,vModiColorbalance.g);
    result.b = colorbalance(result.b,vModiColorbalance.b);
    //cover
    result += texture2D(coverFrame,textureCoordinate);
    
    return result;    
}



/////////////////主程序////////////////////
void main()
{
    vec4 calculatedColor;
    calculatedColor = texture2D(videoFrame,textureCoordinate);
    calculatedColor = beauty(calculatedColor);
    gl_FragColor = calculatedColor;
}
