precision mediump float;

varying highp vec2 textureCoordinate;
uniform sampler2D videoFrame;   //视频预览纹理
uniform sampler2D coverFrame;   //cover纹理
uniform int     _ispreview;     //是否预览图像
uniform int     _middletype;     //0：椭圆形，1：矩形
uniform vec2    _middlepoint;   //焦点中心
uniform vec2    _range;         //焦点区域横向和纵向的半径
uniform int     _type;           //特效类型
uniform int     _depthenable;    //是否打开景深处理,1为打开

vec4 _maskcolor = vec4(1.0,1.0,1.0,0.0);    //预览遮盖色彩
vec2 _framesize = vec2(480.0,320.0);        //完整区域


uniform vec2    _degree;         //角度,景深矩形区域偏转，尚未实现

//模糊
vec4 fuzzy(vec4 color,float intensity)
{   
    vec4 result = color;
    vec2 pTmp;
    
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    result = result * 0.2;
    
    return result;
}

//景深
vec4 depth(vec4 color)
{
    vec4 result = color;
    float fPassRange=80.0;    //椭圆远景过度范围参数
    
    float fModiFuzzy = 0.002;   //离开焦点区域模糊程度参数
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = fuzzy(result,fModiFuzzy);
    
    float distance;
    vec2 tmpCood = vec2(textureCoordinate.y * _framesize.y,textureCoordinate.x * _framesize.x);
    if(_ispreview == 1)     //预览处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            color = normalColor;
        }
    }
    else      //实际处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            result = normalColor;
        }
        
    }
    return result;
}

//dream效果
vec4 dream(vec4 color)
{
    vec4 result = color;
    result *= texture2D(coverFrame,textureCoordinate);
    //景深处理    
    if(_depthenable == 1)
    {
        result = depth(result);
    }
    
    return result;
}

/////////////////主程序////////////////////
void main()
{
    vec4 calculatedColor;
    calculatedColor = texture2D(videoFrame,textureCoordinate);
    calculatedColor = dream(calculatedColor);
    gl_FragColor = calculatedColor;
}
