precision mediump float;

varying highp vec2 textureCoordinate;
uniform sampler2D videoFrame;   //视频预览纹理
uniform sampler2D coverFrame;   //cover纹理
uniform int     _ispreview;     //是否预览图像
uniform int     _middletype;     //0：椭圆形，1：矩形
uniform vec2    _middlepoint;   //焦点中心
uniform vec2    _range;         //焦点区域横向和纵向的半径
uniform int     _type;           //特效类型
uniform int     _depthenable;    //是否打开景深处理,1为打开

vec4 _maskcolor = vec4(1.0,1.0,1.0,0.0);    //预览遮盖色彩
vec2 _framesize = vec2(480.0,320.0);        //完整区域


uniform vec2    _degree;         //角度,景深矩形区域偏转，尚未实现


// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	const float AvgLumR = 0.5;
	const float AvgLumG = 0.5;
	const float AvgLumB = 0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	vec3 conColor = mix(AvgLumin, satColor, con);
	return conColor;
}


//色阶
vec4 modiColorlevel(vec4 color,float inmin,float inmax,float outmin,float outmax)
{
    vec4 result = color;
    vec3 finten;
    if(inmin != inmax)
    {
        finten.r = (result.r  - inmin) / (inmax - inmin);
        finten.g = (result.g  - inmin) / (inmax - inmin);
        finten.b = (result.b  - inmin) / (inmax - inmin);
    }
    result.r = clamp((outmin + finten.r * (outmax - outmin)),outmin,outmax);
    result.g = clamp((outmin + finten.g * (outmax - outmin)),outmin,outmax);
    result.b = clamp((outmin + finten.b * (outmax - outmin)),outmin,outmax);
    return result;
}


//模糊
vec4 fuzzy(vec4 color,float intensity)
{   
    vec4 result = color;
    vec2 pTmp;
    
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x+intensity,textureCoordinate.y-intensity);
    result += texture2D(videoFrame,pTmp);
    pTmp = vec2(textureCoordinate.x-intensity,textureCoordinate.y+intensity);
    result += texture2D(videoFrame,pTmp);
    result = result * 0.2;
    
    return result;
}


//景深
vec4 depth(vec4 color)
{
    vec4 result = color;
    float fPassRange=80.0;    //椭圆远景过度范围参数
    
    float fModiFuzzy = 0.002;   //离开焦点区域模糊程度参数
    //保留正常图像信息
    vec4 normalColor = result;
    //模糊
    result = fuzzy(result,fModiFuzzy);
    
    float distance;
    vec2 tmpCood = vec2(textureCoordinate.y * _framesize.y,textureCoordinate.x * _framesize.x);
    if(_ispreview == 1)     //预览处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(_maskcolor - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            color = normalColor;
        }
    }
    else      //实际处理
    {
        if(_middletype == 1)//椭圆形
        {
            distance = distance(tmpCood,_middlepoint);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);
        }
        else if(_middletype == 0)//矩形
        {
            distance = abs(tmpCood.y - _middlepoint.x);
            result = normalColor +(result - normalColor) * smoothstep(_range.x,_range.x+fPassRange,distance);       
        }
        else
        {
            //处理默认
            result = normalColor;
        }
        
    }
    return result;
}


//艳丽
vec4 lomo(vec4 color)
{
    vec4 result = color;
    float finmax = 230.0/255.0;
    float finmin = 50.0/255.0;
    
    float foutmax = 255.0/255.0;
    float foutmin = 0.0/255.0;
    
    float fModiBrightness=1.0;       //亮度计算控制参数范围0.5-2.0
    float fModiSaturation = 1.30;    //饱和度调整参数
    float fModiContrast = 1.0;      //对比度
    
    //景深处理
    if(_depthenable == 1)
    {
        result = depth(result);
    }
    
    //调整色阶
    result = modiColorlevel(result,finmin,finmax,foutmin,foutmax);
    //调整对比度
    result.rgb =  ContrastSaturationBrightness(result.rgb, fModiBrightness, fModiSaturation, fModiContrast);
    
    //cover
    //result += texture2D(coverFrame,textureCoordinate);

    
    
    return result;
}
/////////////////主程序////////////////////
void main()
{
    vec4 calculatedColor;
    calculatedColor = texture2D(videoFrame,textureCoordinate);
    calculatedColor = lomo(calculatedColor);
    gl_FragColor = calculatedColor;
}
